local channelName = F2PAddonGlobalVars.channelName
local addonChannel = F2PAddonGlobalVars.addonChannel
local thisAddon_version = F2PAddonGlobalVars.addonVersion

local startup = 0
local tmpEventVars = {}
local delay = 1
local counter = 0
local channelListWait


function F2PAddon_OnLoad(self)
	RegisterAddonMessagePrefix(addonChannel)
	RegisterAddonMessagePrefix("f2pq")
	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LIST")
	self:RegisterEvent("CHAT_MSG_CHANNEL_JOIN")
	self:RegisterEvent("CHAT_MSG_CHANNEL_LEAVE")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("PLAYER_LOGOUT")
	self:RegisterEvent("FRIENDLIST_UPDATE")
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("CHAT_MSG_SYSTEM")
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFF2PAddon version "..thisAddon_version.." loaded.|r")
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", F2PChat_FilterP2PTrolls)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", F2PChat_FilterP2PTrolls)
end

function F2PAddon_OnEvent(self, event, ...)
	local a,b,c,d,e,f,g,h,i,j,k = ...
	local prefix, msg, channel, sender = a, b, c, d
	if (event == "PLAYER_LOGIN") then
		--[[]]if F2PAddonGlobalVars.verbose then print("PLAYER_LOGIN") end
		F2PData_BuildDBSkeleton()  --makes sure the DB has the current realm and faction in it
		startup = 1
		--[[]]if F2PAddonGlobalVars.verbose then print("startup = 1") end
		
	elseif (event == "PLAYER_ENTERING_WORLD") and (startup == "done") then
		--[[]]if F2PAddonGlobalVars.verbose then print("PLAYER_ENTERING_WORLD") end
		ListChannelByName(channelName)  --get a new channel listing every time we do a zone change
		
	elseif (event == "CHAT_MSG_CHANNEL_LIST") and (i == channelName) then
		--[[]]if F2PAddonGlobalVars.verbose then print("CHAT_MSG_CHANNEL_LIST") end
		F2PChannel_GetAll(...)  --update the list of who's in the channel
		channelListWait = 3 --set a wait time, in case the channel list is multi part

		
	elseif (event == "CHAT_MSG_CHANNEL_JOIN") and (i == channelName) and (startup == "done") then
		--[[]]if F2PAddonGlobalVars.verbose then print("adding "..b.." to channel and friends.") end
		F2PChannel_AddOne(...)
		F2PFriends_AddJoiner(...)
		F2PChannel_PassOwner(...)
		
	elseif (event == "CHAT_MSG_CHANNEL_LEAVE") and (i == channelName) and (startup == "done")  then
		--[[]]if F2PAddonGlobalVars.verbose then print("removing "..b.." from channel, setting LLIT") end
		F2PChannel_RemoveOne(...)
		F2PData_LLIT_Single(...) 
		--F2PFriends_RemoveLeaver(...)
		
	elseif (event == "CHAT_MSG_ADDON") and (prefix == addonChannel) and (channel == "WHISPER") and ((startup == 4) or (startup == "done"))  then --needs to be to run while waiting for the first pass
		if not string.match(msg, "^f2p=") then --if not a data packet, consider to be a chat message
			F2PChat_FormatIncoming(msg, sender)
		elseif string.match(msg, "^f2p=achid=") then
			F2PAchi_Receive(...)
		elseif string.match(msg, "^f2p=data") then
			if (string.match(msg, "^f2p=datadall:")) then --multi target data packet
				F2PData_Data_Add(msg..sender) --incorporate data packet
				F2PData_SendMyData_ToOne(sender) --send single target packet in return
			elseif  (string.match(msg, "^f2p=datadone:")) then  --single target data packet
				F2PData_Data_Add(msg..sender) --incorporate data packet
			elseif  (string.match(msg, "^f2p=dataalts:")) then
				F2PData_Alts_Add(sender, msg) --incorporate alts data packet
			end
		elseif string.match(msg, "^f2p=ding$") then
			F2PData_RequestWhoLevel(sender)
		end

	elseif (event == "CHAT_MSG_ADDON") and (prefix == "f2pq") and (channel == "WHISPER") then
		F2PQ_ReceiveUpdate(sender,msg)
		
	elseif (event == "FRIENDLIST_UPDATE") then
		F2PFriends_onlineFriendsList_Update()
		
	elseif (event == "ACHIEVEMENT_EARNED") and (startup == "done")  then
		F2PAchi_Send(...)
		
	elseif (event == "UPDATE_BATTLEFIELD_STATUS") and (UnitLevel("player") < 21) then
		F2PQ_SendUpdate()
		
	elseif (event == ("PLAYER_LEVEL_UP")) then --to have everyone request your level from the server via /who
		if (a == 3) then
			F2PData_IDinged()
		end

	elseif (event == ("CHAT_MSG_SYSTEM")) then
		--[[]]if F2PAddonGlobalVars.verbose then print("CHAT_MSG_SYSTEM") end
		for x = 1, #F2PAddonGlobalVars.whoList do
			print(F2PAddonGlobalVars.whoList[x])
			if string.match(a, "%["..F2PAddonGlobalVars.whoList[x].."%]") then
				--[[]]if F2PAddonGlobalVars.verbose then print("who name in whoList") end
				F2PData_ReceiveWhoLevel()
			end
		end
	
			
	elseif (event == "PLAYER_LOGOUT")  and (startup == "done") then
		F2PData_LLIT_Group()
		F2PChannel_PassOwner()
	end
end

function F2PAddon_OnUpdate(self, elapsed)
	counter = counter + elapsed
	if counter >= delay then
		if (startup == 1) and not (tContains({GetChannelList()}, channelName)) then --if we're not yet in the channel,
			--[[]]if F2PAddonGlobalVars.verbose then print("joining channel") end
			JoinTemporaryChannel(channelName) --joined the channel
			counter = 0
		elseif (startup == 1) and (#F2PChat_GetSubscribedChannels(channelName) < 1) then --if no windows are subscribed to display the channel we just joined
			--[[]]if F2PAddonGlobalVars.verbose then print("no chat windows showing channel, subscribing default window") end
			AddChatWindowChannel(DEFAULT_CHAT_FRAME:GetID(), channelName) --make sure a chat window will show the channel
			counter = 0
		elseif  (startup == 1) and (F2PAddonGlobalVars.channelListOK == false) then --if we've sucessfully joined the channel and have it displaying,
			--[[]]if F2PAddonGlobalVars.verbose then print("requesting channel list") end
			ListChannelByName(channelName) --request names of people in channel if we don't have them yet,
			counter = 0
			F2PAddonGlobalVars.channelListOK = false --pause progress until all parts have come back
			startup = 2 -- go into section 2 of the startup process
		
		--Channel list OK, copy it to friends list
		elseif (startup == 2) and (F2PAddonGlobalVars.channelListOK == true) then --once we've got an up to date list of channel members into the addon's global table
			--[[]]if F2PAddonGlobalVars.verbose then print("global channel table updated") end
			--[[]]if F2PAddonGlobalVars.verbose then print("updating friends list") end
			F2PFriends_FLUpdate()  --add people who aren't in the friends list who are in the channel list
			counter = 0
			startup = 3 -- go into section 3 of the startup process
			
		--Friends list OK, update database
		elseif (startup == 3) and (F2PAddonGlobalVars.friendsListOK == true) then --once that's done
			--[[]]if F2PAddonGlobalVars.verbose then print("friends list updated") end
			--[[]]if F2PAddonGlobalVars.verbose then print("adding friend's data to our database, sending our data to friends") end
			--F2PData_Who_Group()  --pull everyone into the database,
			F2PData_SendMyData_ToAll() --provide our own info to everyone,
			startup = 4
			counter = 0
			delay = 3 --wait for other people's data to come back
		elseif (startup == 4) then
			--[[]]if F2PAddonGlobalVars.verbose then print("Checking version") end
			F2PData_GetIfOldVersion() --check to see if anyone has a newer version than us
			startup = "done" --finish startup and return to normal event driven running
			--[[]]if F2PAddonGlobalVars.verbose then print("Startup complete.") end
			delay = 86400 --set delay really high to stop checks on updates
		end
		
		
		
		
		if channelListWait then
			if (F2PAddonGlobalVars.channelListOK == false) and (channelListWait == 0) then
				F2PAddonGlobalVars.channelListOK = true
			elseif (F2PAddonGlobalVars.channelListOK == false) and (channelListWait > 0) then
				--[[]]if F2PAddonGlobalVars.verbose then print("waiting for extra channel list parts") end
				channelListWait = channelListWait - 1
			end
		end
	end
end

