---[[
local channelName = F2PAddonGlobalVars.channelName
local addonChannel = F2PAddonGlobalVars.addonChannel
--]]

local classNamesM = {}
local classNamesF = {}
FillLocalizedClassList(classNamesM ,false)
FillLocalizedClassList(classNamesF ,true)

local subbedChannels = {} -- chat windows that are subbed to show /f2ptwink

local HEX_CLASS_COLORS = {  --saves having to convert them from the r,g,b values bliz uses
	["DEATHKNIGHT"] = "C41F3B",
	["DRUID"] = "FF7D0A",
	["HUNTER"] = "ABD473",
	["MAGE"] = "69CCF0",
	["PALADIN"] = "F58CBA",
	["PRIEST"] = "FFFFFF",
	["ROGUE"] = "FFF569",
	["SHAMAN"] = "0070DE",
	["WARLOCK"] = "9482C9",
	["WARRIOR"] = "C79C6E",
}

---[[
local wowSendChatMessage = SendChatMessage;  --Backup the existing version of SendChatMessage, so it can be called as part of our own version.

function F2PChat_SendChatMessage(msg, chatType, ...)  --Create the hooking fuction
	local a = {...}
	if ( (chatType == "CHANNEL") and (a[#a] == GetChannelName(channelName))) and (not string.match(msg, "^f2p=")) and (IsTrialAccount() or F2PAddonGlobalVars.stillTreatedAsTrialFix) then  --if the message is being sent to our custom channel, you aren't capable of sending normally (not on P2P), and isn't for another addon.
		--print("Message Hooked")
		F2PChat_RouteChatMessage(msg)  --send via addon
		F2PChat_FormatIncoming(msg, F2PAddonGlobalVars.myName) --print sent message
	else  --otherwise send as normal
		wowSendChatMessage(msg, chatType, ...)
	end
end

SendChatMessage = F2PChat_SendChatMessage  --Hook WoW's SendChatMessage, so our version gets called first.
--]]

---[[
function F2PChat_RouteChatMessage(msg)
	if ( not F2PChat_IsBlocked(F2PAddonGlobalVars.myName, 3) ) then
		local f_num = GetNumFriends()
		for y = 1, f_num do
			local name, level, class, loc, connected, status = GetFriendInfo(y)
			if connected and name then
				--print("message sending to "..name)
				ChatThrottleLib:SendAddonMessage("NORMAL", addonChannel, msg, "WHISPER", name)
			end
		end	
	end
end
--]]

function F2PChat_FilterP2PTrolls(self, event, ...)
	local msg, sender, _, _, _, _, _, _, chanName, _, _, _ = ...
	if (event == "CHAT_MSG_CHANNEL") and (chanName == channelName) and (F2PChat_IsBlocked(sender, 3)) then
		return true
	elseif (event == "CHAT_MSG_WHISPER") and (F2PChat_IsBlocked(sender, 3)) then
		return true
	else
		return false
	end
end

---[[
function F2PChat_FormatIncoming(msg, sender)
	if F2PChat_IsBlocked(sender, 3) and (sender == F2PAddonGlobalVars.myName) then  -- if the player is blocked
		msg = "[F2PAddon] Levels 1, 2 and 24 have been prevented from posting to the channel, in order to limit trolling.|r"
	elseif F2PChat_IsBlocked(sender, 3) then  --filters out incoming messages from F2Ps with older addons that don't block source
		return
	end
	
	local senderClassColor = F2PChat_GetSenderClassColorAsHex(sender)
	local senderInfo = F2PInfo_SenderInfo(sender)
	local channelNumber = F2PChat_GetChannelNumber(channelName)
	local channelColor = F2PChat_GetChannelColor(channelNumber)
	local chanHeader = channelNumber.."_ "..channelName
	if F2PAddonGlobalVars.channelHeader then
		chanHeader = F2PAddonGlobalVars.channelHeader
	end
	local header = "|cFF"..channelColor.."["..chanHeader.."] "..senderInfo.."[|cFF"..senderClassColor.."|Hplayer:"..sender.."|h"..sender.."|h|cFF"..channelColor.."] "
	local body = string.gsub(msg,"|r","|cFF"..channelColor) --fixes item links reverting channel color back to white
	local subbedCheck = 0
	for i,v in ipairs(F2PChat_GetSubscribedChannels(channelName)) do
		getglobal("ChatFrame"..v):AddMessage(header..body)
		subbedCheck = 1
	end
	if subbedCheck == 0 then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFF2PAddon has received a message, but has not found a chat window subscribed to display the "..channelName.." channel.|r")
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFRight click a chat window name tab > Settings > Global Channels > ensure that there is a tick before "..channelName..".|r")
	end
end
--]]

--returns the chat frame numbers that are subscribed to the channel
function F2PChat_GetSubscribedChannels(cName)
	local subbedChannels = {}
	for i = 1,NUM_CHAT_WINDOWS do
		if tContains({GetChatWindowChannels(i)}, cName) then
			tinsert(subbedChannels, i)
		end
	end
	return subbedChannels
end

function F2PChat_GetChannelNumber(channelName)
	return GetChannelName(channelName)
end

function F2PChat_GetSenderClassColorAsHex(sender)
	if (sender == F2PAddonGlobalVars.myName) then
		local lname, name = UnitClass("player")
		return HEX_CLASS_COLORS[name]
	end
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, localizedClass, loc, connected, status = GetFriendInfo(y)
		if (sender == name) then
--			print("name = "..name)
--			print("localizedClass = "..localizedClass)
			for n, v in pairs(classNamesM) do
--				print("class name "..n.." = "..v)
				if (v == localizedClass) then
					return HEX_CLASS_COLORS[n]
				end
			end
			for n, v in pairs(classNamesF) do
--				print("class name "..n.." = "..v)
				if v == localizedClass then
					return HEX_CLASS_COLORS[n]
				end
			end
		end
	end
	return "808080"
end


--returns the hex value color of the channel
function F2PChat_GetChannelColor(cNumber)
	local r, g, b
	for i, v in pairs({ChatTypeInfo["CHANNEL"..cNumber]}) do
		for x, y in pairs(v) do
			if x == "r" then
				r = y
			elseif x == "g" then
				g = y
			elseif x == "b" then
				b = y
			end
		end
	end
	return F2PChat_RGBToHex(r, g, b)
end	

function F2PChat_RGBToHex(r, g, b)
   local r = r <= 1 and r >= 0 and r or 1
   local g = g <= 1 and g >= 0 and g or 1
   local b = b <= 1 and b >= 0 and b or 1
   return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

function F2PChat_IsBlocked(sender, minLevel)
	--if security is turned off
	if not F2PAddonGlobalVars.security then
		--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - security is not enabled") end
		return false
	end

	--if this character is trying to send
	local senderLevel
	local myLevel = UnitLevel("player")
	if (sender == F2PAddonGlobalVars.myName) then
		if not F2PChat_LevelOK(myLevel, minLevel) then
			--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - My level "..myLevel..", was too low level") end
			return true
		else
			--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - My level "..myLevel..", was an acceptable level") end
			return false
		end
	end
	
	
	--if the sending character has a level entry in the database
	if F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender]["level"] then
		if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender]["banned"]) then
			--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.." has been banned from the channel") end
			return true
		--if their level is OK
		elseif F2PChat_LevelOK(tonumber(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender]["level"]), minLevel) then
			--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.."'s database entry has an acceptable level: "..tostring(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender]["level"])) end
			return false
		--if their level is not OK
		elseif not F2PChat_LevelOK(tonumber(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender]["level"]), minLevel) and (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender]["level"])  then
			local alts = F2PData_ConfirmedAlts(sender)
			for x = 1, #alts do
				if F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][alts[x]] then
					--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - checking alt "..alts[x]) end
					--check for banned alts
					if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][alts[x]]["banned"]) then
						--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.." has had an alt banned from the channel") end
						return true					
					--check for a confirmed L20 alt
					elseif (tonumber(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][alts[x]]["level"]) == 20) then
						--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.." had a level 20 alt") end
						return false
					end
				end
			end
			--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.."'s database entry has an unacceptable level: "..tostring(senderLevel)) end
			return true
		end
	end
	
	--otherwise fall back to the friends list
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, class, loc, connected, status = GetFriendInfo(y)
		if name == sender then
			senderLevel = tonumber(level)
			--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.."'s friends list entry has level: "..tostring(senderLevel)) end
		end
	end
	if not senderLevel then
		senderLevel = 0
		--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - No level found for "..sender) end
	end
	if F2PChat_LevelOK(senderLevel, minLevel) then
		--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.." was of an acceptable level") end
		return false
	end
	--[[]]if F2PAddonGlobalVars.verbose then print("F2PChat_IsBlocked - "..sender.." was of an unknown level") end
	return true
end

function F2PChat_LevelOK(playerLevel, minLevel)
	if (playerLevel < minLevel) or (playerLevel == 24) then
		return false
	else
		return true
	end
end