
local channelName = F2PAddonGlobalVars.channelName
local addonChannel = F2PAddonGlobalVars.addonChannel


--#############################################################
--# Form data packets containing this character's information #
--#############################################################

function F2PData_OutgoingDataPacket()
	
	local _, class = UnitClass(F2PAddonGlobalVars.myName)
	c1, c2 = string.match(class, "^(.)(.+)$")
	class = c1..string.lower(c2)
	
	local level = UnitLevel(F2PAddonGlobalVars.myName)

	local _, race = UnitRace(F2PAddonGlobalVars.myName)
	if (race == "Scourge") then
	 race = "Forsaken"
	end
	local gender = UnitSex(F2PAddonGlobalVars.myName)
	if gender == 3 then
		gender = "female"
	elseif gender == 2 then
		gender = "male"
	end
	local talents = {}
	for x = 1, 3 do
		local id, localizedName, _, _, points, _, _, _ = GetTalentTabInfo(x, nil, nil, nil)
		talents[points] = id
	end
	local spec = (talents[table.maxn(talents)])

	local trial
	if IsTrialAccount() then
		trial = 1
	else
		trial = 0
	end
		
	local xpLocked = IsXPUserDisabled()
	if xpLocked then
		xplocked = "1"
	else
		xpLocked = "0"
	end

	local data = {
		["class"] = class,
		["level"] = level,
		["race"] = race,
		["gender"] = gender,
		["spec"] = spec,
		["trial"] = trial,
		["xpLocked"] = xpLocked,
		["version"] = F2PAddonGlobalVars.addonVersion,
	}

	local dataMsg = ""
	
	for i, v in pairs(data) do
		dataMsg = dataMsg..i..","..tostring(v)..":"
	end

	return dataMsg
end

function F2PData_OutgoingAltsPacket()
	local altsMsg = "f2p=dataalts:"
	--[[]]if F2PAddonGlobalVars.verbose then print("My alts are:") end
	--[[]]if F2PAddonGlobalVars.verbose then print(table.concat(F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm], ",")) end
	altsMsg = altsMsg..table.concat(F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm], ",")
	return altsMsg
end

--#########################################
--# Send data packets to other characters #
--#########################################

function F2PData_SendMyData_ToAll()
	--[[]]if F2PAddonGlobalVars.verbose then print("Sending to everyone: ".."f2p=datadall:"..F2PData_OutgoingDataPacket()) end
	F2PData_RouteDataPacket("f2p=datadall:"..F2PData_OutgoingDataPacket())
	if (#F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm] > 1) then --if this account has alts
		--[[]]if F2PAddonGlobalVars.verbose then print("Sending to everyone: "..F2PData_OutgoingAltsPacket()) end
		F2PData_RouteDataPacket(F2PData_OutgoingAltsPacket())
	end
end

function F2PData_SendMyData_ToOne(sender)
	--[[]]if F2PAddonGlobalVars.verbose then print("Sending to "..sender..": ".."f2p=datadone:"..F2PData_OutgoingDataPacket()) end
	ChatThrottleLib:SendAddonMessage("NORMAL", addonChannel, "f2p=datadone:"..F2PData_OutgoingDataPacket(), "WHISPER", sender)
	if (#F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm] > 1) then --if this account has alts
		--[[]]if F2PAddonGlobalVars.verbose then print("Sending to "..sender..": "..F2PData_OutgoingAltsPacket()) end
		ChatThrottleLib:SendAddonMessage("NORMAL", addonChannel, F2PData_OutgoingAltsPacket(), "WHISPER", sender)
	end
end

--#########################
--# Data routing function #
--#########################

---[[
function F2PData_RouteDataPacket(msg)
	local f_num = GetNumFriends()
	for y = 1, f_num do
		local name, level, class, loc, connected, status = GetFriendInfo(y)
		if connected and name then
			--[[]]if F2PAddonGlobalVars.verbose then print("sending data packet to "..name) end
			ChatThrottleLib:SendAddonMessage("NORMAL", addonChannel, msg, "WHISPER", name)
		end
	end	
end
--]]

--####################################################################
--# Special case to have everyone request your level from the server #
--####################################################################

function F2PData_IDinged()
	--[[]]if F2PAddonGlobalVars.verbose then print("sending ding announcement") end
	F2PData_RouteDataPacket("f2p=ding")
end

function F2PData_RequestWhoLevel(sender)
	--[[]]if F2PAddonGlobalVars.verbose then print("received ding announcement from "..sender..", adding name to whoList and requesting /who.") end
	SetWhoToUI(0)
	SendWho('n-"'..sender..'"')
	tinsert(F2PAddonGlobalVars.whoList, sender)
end

function F2PData_ReceiveWhoLevel()
	--[[]]if F2PAddonGlobalVars.verbose then print("Who received.") end
	local numResults, totalCount = GetNumWhoResults()
	for x = 1, numResults do
		local name, guild, level, race, class, zone, classFileName, sex = GetWhoInfo(x)
		if tContains(F2PAddonGlobalVars.whoList, name) then
			if F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][name] then
				--[[]]if F2PAddonGlobalVars.verbose then print("Updating database entry for "..name.." with level "..level) end
				F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][name]["level"] = level
				for y = 1, #F2PAddonGlobalVars.whoList do
					if F2PAddonGlobalVars.whoList[y] == name then
						tremove(F2PAddonGlobalVars.whoList, y)
					end
				end
			end
		end
	end
end



--#####################################
--# Add incoming data to the database #
--#####################################

function F2PData_Data_Add(...)
	local input = ...
	--print("Recieved  "..input)
	dataName = string.match(input, ":([^:]+)$")
	--print(dataName)
	
	--if not F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][dataName] then
	--	tinsert(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction], {dataName})
	--end

	if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][dataName] == nil) then
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][dataName] = {}
	end
	--[[]]if F2PAddonGlobalVars.verbose then print("------------") end
	--[[]]if F2PAddonGlobalVars.verbose then print("Adding data for "..dataName..":") end

	
	for i, v in string.gmatch(input, ":([^,]+),([^:]+)") do
		--[[]]if F2PAddonGlobalVars.verbose then print(i..", "..v) end
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][dataName][i] = v
	end	
end

function F2PData_Alts_Add(sender, input)

	if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender] == nil) then
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender] = {}
	end
	
	local output = string.match(input, "^f2p=dataalts:([^:]+)$")
	
	--[[]]if F2PAddonGlobalVars.verbose then print("alts of "..sender.." are: "..output) end
	F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][sender].Alts = output
end

--##########################
--# Log out time recording #
--##########################


function F2PData_LLIT_Single(...)	--Last Logged In Time
	local a,b,c,d,e,f,g,h,i = ...
	currentTime = date("y%yM%md%dh%Hm%M")
	currentTime = string.gsub(currentTime,"h%d%d", "h"..string.format("%02d", GetGameTime()))
	if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][b] == nil) then
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][b] = {}
	end
	F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][b]["LLIT"] = currentTime
end

function F2PData_LLIT_Group()	--Last Logged In Time
	currentTime = date("y%yM%md%dh%Hm%M")
	currentTime = string.gsub(currentTime,"h%d%d", "h"..string.format("%02d", GetGameTime()))
	for i, v in pairs(F2PAddonGlobalVars.onlineFriendsList) do
		if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][v] == nil) then
			F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][v] = {}
		end
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][v]["LLIT"] = currentTime
	end
end


--#########################################
--# show which alts belong to a character #
--#########################################

function F2PData_ConfirmedAlts(name)
	local confirmedList = {}
	if F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][name] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][name].Alts then
		for n in string.gmatch(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][name].Alts, "([^,]+)") do
			--print(n)
			if F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][n] and F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][n].Alts then
				if string.match(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][n].Alts, name) then
					tinsert(confirmedList, n)
				end
			end
		end
	end
	return confirmedList
end

--###########################
--# Compare version numbers #
--###########################

function F2PData_GetIfOldVersion()
	local isOld = 0
	local ourVersion = F2PAddonGlobalVars.addonVersion
	local otherVersion
	for i, v in pairs(F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction]) do
		if F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][i].version then
			otherVersion = F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction][i]["version"]
			if F2PData_CompareVersions(ourVersion, otherVersion) then
				isOld = 1
				ourVersion = otherVersion
			end
		end
	end
	if isOld == 1 then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFF00FFYour version of F2PAddon is outdated.  "..ourVersion.." is the current release.")
	end
end

function F2PData_CompareVersions(ours, others)

	local a1 = string.match(ours, "^([%d]+)\.")
	local a2 = string.match(ours, "\.([%d]+)\.")
	local a3 = string.match(ours, "[%d]+\.[%d]+\.([^b]+)b?$")

	local b1 = string.match(others, "^([%d]+)\.")
	local b2 = string.match(others, "\.([%d]+)\.")
	local b3 = string.match(others, "[%d]+\.[%d]+\.([^b]+)b?$")

	if tonumber(a1) < tonumber(b1) then
		return true
	elseif tonumber(a1) == tonumber(b1) then
		if tonumber(a2) < tonumber(b2) then
			return true
		elseif tonumber(a2) == tonumber(b2) then
			if tonumber(a3) < tonumber(b3) then
				return true
			elseif tonumber(a3) == tonumber(b3) then
				return false
			else
				return false
			end
		else
			return false
		end
	else
		return false
	end
	
end


--###################################
--# Set up basic database structure #
--###################################

function F2PData_BuildDBSkeleton()
	if (F2PAddon_Variables == nil) then --if the complete table is blank, as when the addon is first installed
		F2PAddon_Variables = {			--make this it's basic structure
			["Alts"] = {
				[F2PAddonGlobalVars.thisRealm] = {},
			},
			["Realms"] = {
				[F2PAddonGlobalVars.thisRealm] = {
					[F2PAddonGlobalVars.myFaction] = {},
				},
			},
		}
	end

	if (F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm] == nil) then --if the Alts data for this realm doesn't exist
		F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm] = {}		 --add a new blank table for it
	end
	
	if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm] == nil) then --if the data for this realm doesn't exist,
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm] = {}		 --add a new blank table for it
	end

	if (F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction] == nil) then --if the data for this faction doesn't exist for this realm,
		F2PAddon_Variables.Realms[F2PAddonGlobalVars.thisRealm][F2PAddonGlobalVars.myFaction] = {}		 --add a new blank table for it
	end

	for i, v in pairs(F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm]) do --clears out the alts data if the database contains the old 'name = false' format
		if i == F2PAddonGlobalVars.myName and v == false then
			F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm] = {}
		end
	end
		
	if not tContains(F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm], F2PAddonGlobalVars.myName) then
		tinsert(F2PAddon_Variables.Alts[F2PAddonGlobalVars.thisRealm], F2PAddonGlobalVars.myName)
	end
	
end